//
//  AFGitHubGitObject.m
//
//  Copyright (c) 2012 Atsushi Nagase (http://ngs.io/)
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.

#import "AFGitHubGitObject.h"
#import "AFGitHubGlobal.h"
#import "AFGitHubCommit.h"
#import "AFGitHubTag.h"
#import "AFGitHubBlob.h"
#import "AFGitHubTree.h"

@implementation AFGitHubGitObject

+ (AFGitHubGitObject *)gitObjectWithDictionary:(NSDictionary *)dictionary {
  if(![dictionary isKindOfClass:[NSDictionary class]])
    return nil;
  NSString *type = dictionary[@"type"];
  if([type isEqualToString:@"tag"])
    return [[AFGitHubTag alloc] initWithDictionary:dictionary];
  if([type isEqualToString:@"commit"])
    return [[AFGitHubCommit alloc] initWithDictionary:dictionary];
  if([type isEqualToString:@"blob"])
    return [[AFGitHubBlob alloc] initWithDictionary:dictionary];
  if([type isEqualToString:@"tree"])
    return [[AFGitHubTree alloc] initWithDictionary:dictionary];
  return nil;
}

- (id)initWithDictionary:(NSDictionary *)dictonary {
  if(self = [super init]) {
    self.SHA = dictonary[@"sha"];
    self.URL = [NSURL URLWithString:dictonary[@"url"]];
  }
  return self;
}

#pragma mark - NSCopying

- (id)copyWithZone:(NSZone *)zone {
  id copy = [[self.class alloc] init];
  [copy setSHA:self.SHA];
  [copy setURL:self.URL];
  return copy;
}

#pragma mark - NSCoding

- (id)initWithCoder:(NSCoder *)aDecoder {
  if(self = [self init]) {
    self.SHA = [aDecoder decodeObjectForKey:@"sha"];
    self.URL = [aDecoder decodeObjectForKey:@"url"];
  }
  return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder {
  [aCoder encodeObject:self.URL forKey:@"url"];
  [aCoder encodeObject:self.SHA forKey:@"sha"];
}

@end
